@@ -77,7 +77,7 @@ def detect_language() -> str:
 
 
 @click.group()
-@click.version_option(version="2.0.4")
+@click.version_option(version="2.1.2")
 def cli():
     """Engine - AI-powered code generation."""
     pass
@@ -89,7 +89,7 @@ def cli():
 
 @cli.command()
 @click.option("--language", "-l", type=click.Choice(["python", "typescript", "auto"]), default="auto")
-@click.option("--embed", "-e", is_flag=True, help="Build embeddings for RAG (requires OPENAI_API_KEY)")
+@click.option("--embed", "-e", is_flag=True, help="Build embeddings for RAG (semantic search)")
 @click.option("--force", "-f", is_flag=True, help="Force full re-index (ignore cache)")
 def index(language: str, embed: bool, force: bool):
     """Index the current project (incrementally by default)."""
@@ -144,16 +144,13 @@ def index(language: str, embed: bool, force: bool):
     
     # Build embeddings if requested
     if embed:
-        import os
-        api_key = os.getenv('OPENAI_API_KEY')
-        
-        if not api_key:
-            console.print("[yellow]⚠ OPENAI_API_KEY not set. Skipping embeddings.[/yellow]")
-            console.print("[dim]Set OPENAI_API_KEY to enable RAG-based context search.[/dim]")
-            return
-        
         try:
-            from engine.embeddings import RAGIndexer
+            from engine.embeddings import RAGIndexer, HAS_EMBEDDINGS
+            
+            if not HAS_EMBEDDINGS:
+                console.print("[yellow]⚠ sentence-transformers not installed.[/yellow]")
+                console.print("[dim]Run: pip install sentence-transformers[/dim]")
+                return
             
             console.print("\n[blue]Building embeddings for RAG...[/blue]")
             
@@ -164,7 +161,7 @@ def index(language: str, embed: bool, force: bool):
             ) as progress:
                 task = progress.add_task("Generating embeddings...", total=None)
                 
-                indexer = RAGIndexer(cwd, api_key)
+                indexer = RAGIndexer(cwd)
                 count = indexer.build_from_index(index_data, show_progress=False)
             
             console.print(f"[green]✓ Embedded {count} items for semantic search[/green]")
@@ -172,7 +169,7 @@ def index(language: str, embed: bool, force: bool):
         
         except ImportError as e:
             console.print(f"[yellow]⚠ Missing dependencies: {e}[/yellow]")
-            console.print("[dim]Run: pip install openai numpy[/dim]")
+            console.print("[dim]Run: pip install sentence-transformers numpy[/dim]")
         except Exception as e:
             console.print(f"[red]✗ Embedding failed: {e}[/red]")
 
@@ -206,11 +203,7 @@ def status():
         # Check RAG status
         vectors_db = cwd / ".engine" / "vectors.db"
         if vectors_db.exists():
-            import os
-            if os.getenv('OPENAI_API_KEY'):
-                console.print("  • [green]RAG: Enabled[/green] (semantic search)")
-            else:
-                console.print("  • [yellow]RAG: Embeddings exist but OPENAI_API_KEY not set[/yellow]")
+            console.print("  • [green]RAG: Enabled[/green] (semantic search)")
         else:
             console.print("  • [dim]RAG: Not enabled (run 'engine index --embed')[/dim]")
     else:
@@ -242,18 +235,24 @@ def status():
 # ============================================================
 
 @cli.command()
-@click.argument("task")
+@click.argument("task", required=False, default=None)
 @click.option("--file", "-f", multiple=True, help="Files to focus on")
 @click.option("--dry-run", is_flag=True, help="Show what would be generated without applying")
 @click.option("--no-stream", is_flag=True, help="Disable streaming output")
 def generate(task: str, file: tuple, dry_run: bool, no_stream: bool):
-    """Generate code for a task."""
+    """Generate code for a task. Run without arguments for guided suggestions."""
     # Check license
     if not is_license_stored():
         console.print("[red]No license found.[/red]")
         console.print("Run 'engine license activate <key>' or 'engine license trial <email>'")
         sys.exit(1)
     
+    # If no task provided, show onboarding/suggestions
+    if not task:
+        task = _show_task_suggestions()
+        if not task:
+            return
+    
     # Load index
     index_data = get_project_index()
     
@@ -264,9 +263,42 @@ def generate(task: str, file: tuple, dry_run: bool, no_stream: bool):
     relevant_files = assembler.get_relevant_files(task)
     
     if dry_run:
-        console.print("\n[yellow]Dry run - context that would be sent:[/yellow]")
-        console.print(Panel(context[:2000] + "..." if len(context) > 2000 else context))
-        console.print(f"\n[dim]Relevant files: {', '.join(relevant_files[:5])}[/dim]")
+        console.print("\n[bold yellow]🔍 Dry Run - Preview Mode[/bold yellow]\n")
+        
+        # Show relevant files that would be analyzed
+        console.print("[cyan]Files that will be analyzed:[/cyan]")
+        for f in relevant_files[:10]:
+            if Path(f).exists():
+                console.print(f"  [green]●[/green] {f} [dim](exists)[/dim]")
+            else:
+                console.print(f"  [yellow]○[/yellow] {f} [dim](new file)[/dim]")
+        
+        if len(relevant_files) > 10:
+            console.print(f"  [dim]... and {len(relevant_files) - 10} more files[/dim]")
+        
+        # Analyze task to predict what will happen
+        console.print("\n[cyan]Task analysis:[/cyan]")
+        task_lower = task.lower()
+        if any(word in task_lower for word in ["fix", "bug", "correct", "repair", "change", "update"]):
+            console.print("  [yellow]⚠[/yellow]  Mode: [yellow]BUG FIX[/yellow] - minimal changes only")
+            console.print("     Engine will modify only the lines needed to fix the issue")
+        elif any(word in task_lower for word in ["add", "create", "implement", "build"]):
+            console.print("  [green]●[/green]  Mode: [green]FEATURE[/green] - new code generation")
+            console.print("     Engine may create new files or extend existing ones")
+        elif any(word in task_lower for word in ["refactor", "reorganize", "restructure"]):
+            console.print("  [blue]●[/blue]  Mode: [blue]REFACTOR[/blue] - code restructuring")
+            console.print("     Engine may modify multiple files")
+        else:
+            console.print("  [dim]●[/dim]  Mode: [dim]GENERAL[/dim]")
+        
+        # Show context preview
+        console.print("\n[cyan]Context preview (first 1500 chars):[/cyan]")
+        console.print(Panel(context[:1500] + "..." if len(context) > 1500 else context, 
+                           title="Context to be sent", border_style="dim"))
+        
+        # Show what will happen next
+        console.print("\n[dim]Run without --dry-run to generate and apply changes.[/dim]")
+        console.print("[dim]You'll see a preview of changes before applying.[/dim]")
         return
     
     try:
@@ -284,8 +316,7 @@ def generate(task: str, file: tuple, dry_run: bool, no_stream: bool):
         sys.exit(1)
     
     except UsageLimitError as e:
-        console.print(f"[red]Usage limit reached:[/red] {e.limit_name} ({e.current}/{e.limit})")
-        console.print(f"[blue]Upgrade at:[/blue] {e.upgrade_url}")
+        _show_upgrade_prompt(e)
         sys.exit(1)
     
     except APIError as e:
@@ -293,6 +324,134 @@ def generate(task: str, file: tuple, dry_run: bool, no_stream: bool):
         sys.exit(1)
 
 
+def _show_task_suggestions() -> str:
+    """Show interactive task suggestions for onboarding."""
+    console.print("\n[bold cyan]🚀 What would you like to build?[/bold cyan]\n")
+    
+    # Detect project type for relevant suggestions
+    cwd = Path.cwd()
+    has_fastapi = any(cwd.glob("**/main.py")) or any(cwd.glob("**/app.py"))
+    has_django = any(cwd.glob("**/manage.py"))
+    has_react = (cwd / "package.json").exists()
+    
+    # Build suggestions based on project type
+    suggestions = []
+    
+    if has_fastapi or has_django or (not has_react):
+        suggestions.extend([
+            ("1", "Add JWT authentication with login/register endpoints", "🔐"),
+            ("2", "Create a new model with full CRUD API endpoints", "📦"),
+            ("3", "Add Stripe payment webhook handler", "💳"),
+            ("4", "Add rate limiting middleware", "🛡️"),
+            ("5", "Create comprehensive tests for existing endpoints", "🧪"),
+        ])
+    
+    if has_react:
+        suggestions.extend([
+            ("1", "Add user authentication with login/signup forms", "🔐"),
+            ("2", "Create a data table component with sorting and filtering", "📊"),
+            ("3", "Add a settings page with form validation", "⚙️"),
+            ("4", "Create a dashboard with charts and metrics", "📈"),
+            ("5", "Add dark mode toggle with theme context", "🌙"),
+        ])
+    
+    if not suggestions:
+        suggestions = [
+            ("1", "Add user authentication", "🔐"),
+            ("2", "Create a new model with CRUD operations", "📦"),
+            ("3", "Add API endpoints for a new feature", "🔌"),
+            ("4", "Create unit tests for existing code", "🧪"),
+            ("5", "Add input validation and error handling", "✅"),
+        ]
+    
+    # Display suggestions
+    for num, desc, icon in suggestions:
+        console.print(f"  [cyan]{num}[/cyan]  {icon}  {desc}")
+    
+    console.print(f"\n  [dim]c[/dim]     ✏️   Enter custom task")
+    console.print(f"  [dim]q[/dim]     🚪  Quit\n")
+    
+    # Get user choice
+    choice = click.prompt(
+        "Choose an option",
+        type=str,
+        default="c"
+    ).strip().lower()
+    
+    if choice == "q":
+        console.print("[dim]Cancelled.[/dim]")
+        return None
+    
+    if choice == "c":
+        task = click.prompt("Describe what you want to build")
+        return task
+    
+    # Find matching suggestion
+    for num, desc, _ in suggestions:
+        if choice == num:
+            console.print(f"\n[green]Selected:[/green] {desc}\n")
+            return desc
+    
+    # Treat as custom input if no match
+    return choice
+
+
+def _show_upgrade_prompt(e):
+    """Show a helpful upgrade prompt when usage limit is hit."""
+    console.print("\n[bold yellow]⚠️  Generation limit reached[/bold yellow]\n")
+    
+    # Show current usage
+    console.print(f"  {e.limit_name}: {e.current}/{e.limit} used\n")
+    
+    # Show upgrade benefits
+    console.print("[bold]Upgrade to Engine Pro ($39/mo):[/bold]")
+    console.print("  ✓ 200 generations per month")
+    console.print("  ✓ Instant rollback safety net")
+    console.print("  ✓ Full tool-use verification")
+    console.print("  ✓ Priority support\n")
+    
+    console.print("[bold]Or Engine Team ($119/mo):[/bold]")
+    console.print("  ✓ 500 generations per month")
+    console.print("  ✓ Shared pattern library")
+    console.print("  ✓ Team analytics dashboard")
+    console.print("  ✓ Everything in Pro\n")
+    
+    console.print(f"[blue]→ Upgrade now:[/blue] {e.upgrade_url}")
+    console.print("[dim]  Or run: engine upgrade[/dim]\n")
+
+
+def _show_usage_meter(client):
+    """Show usage meter after generation."""
+    try:
+        usage = client.get_usage()
+        
+        # Get stored license for tier info
+        stored = get_stored_license()
+        tier = stored.tier if stored else "trial"
+        
+        if tier == "trial":
+            # Trial: show X/5 with emphasis
+            remaining = usage.monthly_limit - usage.monthly_used
+            if remaining <= 2:
+                console.print(f"[yellow]📊 Trial: {usage.monthly_used}/{usage.monthly_limit} generations used ({remaining} remaining)[/yellow]")
+                if remaining == 0:
+                    console.print("[dim]   Run 'engine upgrade' to continue generating[/dim]")
+            else:
+                console.print(f"[dim]📊 Trial: {usage.monthly_used}/{usage.monthly_limit} generations ({remaining} remaining)[/dim]")
+        else:
+            # Paid tier: show monthly usage
+            pct = (usage.monthly_used / usage.monthly_limit * 100) if usage.monthly_limit > 0 else 0
+            if pct >= 80:
+                console.print(f"[yellow]📊 Usage: {usage.monthly_used}/{usage.monthly_limit} this month ({pct:.0f}%)[/yellow]")
+            else:
+                console.print(f"[dim]📊 Usage: {usage.monthly_used}/{usage.monthly_limit} this month[/dim]")
+    except Exception:
+        # Silently skip if can't get usage
+        pass
+    
+    console.print()  # Add spacing
+
+
 def _generate_with_stream(client, task: str, context: str, relevant_files: list[str]):
     """Generate code with streaming output."""
     from rich.live import Live
@@ -355,11 +514,30 @@ def _generate_with_stream(client, task: str, context: str, relevant_files: list[
     if tool_calls_made > 0:
         stats_line += f" [dim](verified with {tool_calls_made} tool calls)[/dim]"
     console.print(stats_line)
-    console.print(f"[dim]Tokens: {input_tokens} in / {output_tokens} out[/dim]\n")
+    console.print(f"[dim]Tokens: {input_tokens} in / {output_tokens} out[/dim]")
+    
+    # Show usage meter
+    _show_usage_meter(client)
     
     # Apply changes
     if files_generated:
-        if click.confirm("Apply these changes?"):
+        # Offer preview option
+        choice = click.prompt(
+            "Apply changes?",
+            type=click.Choice(['y', 'n', 'p'], case_sensitive=False),
+            default='y',
+            show_choices=True,
+            prompt_suffix=" (y=yes, n=no, p=preview): "
+        )
+        
+        if choice.lower() == 'p':
+            _preview_changes(full_content)
+            if click.confirm("Apply these changes?"):
+                _apply_changes(full_content, task=task)
+                console.print("[green]✓ Changes applied[/green]")
+            else:
+                console.print("[yellow]Changes not applied[/yellow]")
+        elif choice.lower() == 'y':
             _apply_changes(full_content, task=task)
             console.print("[green]✓ Changes applied[/green]")
         else:
@@ -386,7 +564,10 @@ def _generate_no_stream(client, task: str, context: str, relevant_files: list[st
     
     # Display result
     console.print(f"\n[green]✓ Generated {len(response.files_generated)} file(s)[/green]")
-    console.print(f"[dim]Tokens: {response.input_tokens} in / {response.output_tokens} out[/dim]\n")
+    console.print(f"[dim]Tokens: {response.input_tokens} in / {response.output_tokens} out[/dim]")
+    
+    # Show usage meter
+    _show_usage_meter(client)
     
     # Show generated code
     console.print(Panel(
@@ -396,7 +577,22 @@ def _generate_no_stream(client, task: str, context: str, relevant_files: list[st
     
     # Apply changes
     if response.files_generated:
-        if click.confirm("\nApply these changes?"):
+        choice = click.prompt(
+            "Apply changes?",
+            type=click.Choice(['y', 'n', 'p'], case_sensitive=False),
+            default='y',
+            show_choices=True,
+            prompt_suffix=" (y=yes, n=no, p=preview): "
+        )
+        
+        if choice.lower() == 'p':
+            _preview_changes(response.content)
+            if click.confirm("Apply these changes?"):
+                _apply_changes(response.content, task=task)
+                console.print("[green]✓ Changes applied[/green]")
+            else:
+                console.print("[yellow]Changes not applied[/yellow]")
+        elif choice.lower() == 'y':
             _apply_changes(response.content, task=task)
             console.print("[green]✓ Changes applied[/green]")
         else:
@@ -558,23 +754,36 @@ def activate(key: str):
 @license.command()
 @click.argument("email")
 def trial(email: str):
-    """Start a free trial."""
-    console.print("[blue]Creating trial...[/blue]")
+    """Start a free trial with email verification."""
+    console.print("[blue]Requesting verification code...[/blue]")
     
     try:
         client = get_api_client()
-        info = client.create_trial(email)
+        
+        # Step 1: Request verification code
+        result = client.request_trial(email)
+        
+        console.print(f"[green]✓ Verification code sent to {email}[/green]")
+        console.print("[dim]Check your inbox (and spam folder)[/dim]\n")
+        
+        # Step 2: Prompt for code
+        code = click.prompt("Enter the 6-digit code", type=str)
+        
+        console.print("[blue]Verifying code...[/blue]")
+        
+        # Step 3: Verify and create trial
+        info = client.verify_trial(email, code.strip())
         
         # Save locally
         save_license(info.license_key, info.tier, info.limits)
         
-        console.print(f"[green]✓ Trial activated for {email}![/green]")
+        console.print(f"\n[green]✓ Trial activated for {email}![/green]")
         console.print(f"\nYou now have [bold]5 generations[/bold] to try Pro features:")
         console.print("  • engine plan - Multi-task planning")
         console.print("  • Extended context limits")
         console.print("  • Chat history")
         console.print(f"\n[dim]License key: {info.license_key}[/dim]")
-        console.print("\n[blue]Upgrade at:[/blue] https://engine.dev/pricing")
+        console.print("\n[blue]Upgrade at:[/blue] https://enginecli.dev/pricing")
     
     except APIError as e:
         console.print(f"[red]Trial creation failed:[/red] {e}")
@@ -626,6 +835,53 @@ def deactivate():
             console.print(f"[red]Deactivation failed:[/red] {e}")
 
 
+# ============================================================
+# Upgrade Command
+# ============================================================
+
+@cli.command()
+@click.option("--tier", "-t", type=click.Choice(["pro", "team"]), default=None,
+              help="Tier to upgrade to")
+def upgrade(tier: str):
+    """Upgrade your Engine subscription."""
+    import webbrowser
+    
+    base_url = get_api_url().replace("/v1", "")
+    
+    if not tier:
+        console.print("\n[bold cyan]🚀 Upgrade Engine[/bold cyan]\n")
+        console.print("[bold]Pro[/bold] - $39/month")
+        console.print("  ✓ 200 generations/month")
+        console.print("  ✓ Instant rollback")
+        console.print("  ✓ Full tool-use verification")
+        console.print("  ✓ Priority support\n")
+        
+        console.print("[bold]Team[/bold] - $119/month")
+        console.print("  ✓ 500 generations/month")
+        console.print("  ✓ Shared pattern library")
+        console.print("  ✓ Team analytics dashboard")
+        console.print("  ✓ Everything in Pro\n")
+        
+        tier = click.prompt(
+            "Which tier",
+            type=click.Choice(["pro", "team"]),
+            default="pro"
+        )
+    
+    # Open checkout page
+    checkout_url = f"{base_url}/#pricing"
+    console.print(f"\n[blue]Opening checkout...[/blue]")
+    console.print(f"[dim]{checkout_url}[/dim]\n")
+    
+    try:
+        webbrowser.open(checkout_url)
+        console.print("[green]✓ Checkout page opened in browser[/green]")
+        console.print("[dim]Complete payment to activate your subscription.[/dim]")
+    except Exception:
+        console.print(f"[yellow]Could not open browser. Visit:[/yellow]")
+        console.print(f"  {checkout_url}")
+
+
 # ============================================================
 # Analytics Commands
 # ============================================================
@@ -1046,17 +1302,84 @@ def config_list():
 # Helper Functions
 # ============================================================
 
+def _preview_changes(content: str):
+    """Preview file changes with diffs before applying."""
+    import difflib
+    
+    changes = _extract_file_changes(content)
+    
+    if not changes:
+        console.print("[yellow]No changes to preview.[/yellow]")
+        return
+    
+    console.print("\n[bold cyan]═══ PREVIEW ═══[/bold cyan]\n")
+    
+    for file_path, new_content in changes:
+        path = Path(file_path)
+        
+        # Delete marker
+        if new_content == '__DELETE_FILE__':
+            console.print(f"[red]─── DELETE: {file_path} ───[/red]")
+            continue
+        
+        # New file
+        if not path.exists():
+            console.print(f"[cyan]─── NEW: {file_path} ───[/cyan]")
+            # Show first 20 lines
+            lines = new_content.splitlines()[:20]
+            for line in lines:
+                console.print(f"[green]+ {line}[/green]")
+            if len(new_content.splitlines()) > 20:
+                console.print(f"[dim]... ({len(new_content.splitlines()) - 20} more lines)[/dim]")
+            console.print()
+            continue
+        
+        # Modified file - show diff
+        console.print(f"[yellow]─── MODIFIED: {file_path} ───[/yellow]")
+        old_content = path.read_text()
+        
+        diff = difflib.unified_diff(
+            old_content.splitlines(keepends=True),
+            new_content.splitlines(keepends=True),
+            fromfile=f"a/{file_path}",
+            tofile=f"b/{file_path}",
+            lineterm=""
+        )
+        
+        diff_lines = list(diff)
+        if not diff_lines:
+            console.print("[dim]No changes[/dim]")
+        else:
+            # Show max 30 diff lines
+            for i, line in enumerate(diff_lines[:30]):
+                line = line.rstrip('\n')
+                if line.startswith('+') and not line.startswith('+++'):
+                    console.print(f"[green]{line}[/green]")
+                elif line.startswith('-') and not line.startswith('---'):
+                    console.print(f"[red]{line}[/red]")
+                elif line.startswith('@@'):
+                    console.print(f"[cyan]{line}[/cyan]")
+                else:
+                    console.print(f"[dim]{line}[/dim]")
+            
+            if len(diff_lines) > 30:
+                console.print(f"[dim]... ({len(diff_lines) - 30} more lines)[/dim]")
+        
+        console.print()
+    
+    console.print("[bold cyan]═══ END PREVIEW ═══[/bold cyan]\n")
+
+
 def _apply_changes(content: str, task: str = "code generation"):
     """
     Apply generated code to files.
     
     Creates a snapshot before applying changes for rollback safety.
-    
-    Supports two formats:
-    1. New format: <<<<< FILE: path >>>>> ... <<<<< END FILE >>>>>
-    2. Legacy format: FILE: path followed by code block
+    Includes: preview, auto-install deps, migration hints, git commit.
     """
     import re
+    import subprocess
+    import difflib
     from engine.rollback import RollbackManager
     
     # Extract all file changes first
@@ -1072,23 +1395,150 @@ def _apply_changes(content: str, task: str = "code generation"):
     snapshot = manager.create_snapshot(file_paths, task)
     console.print(f"[dim]Snapshot created: {snapshot.id}[/dim]")
     
+    # Track changes for post-apply actions
+    new_deps = []
+    schema_changed = False
+    modified_files = []
+    new_files = []
+    
     # Apply changes
     for file_path, file_content in changes:
+        path = Path(file_path)
+        
         # Check for delete marker
         if file_content == '__DELETE_FILE__':
-            path = Path(file_path)
             if path.exists():
                 path.unlink()
                 console.print(f"  [red]✗ Deleted[/red] {file_path}")
             continue
         
+        # Track if new or modified
+        is_new = not path.exists()
+        if is_new:
+            new_files.append(file_path)
+        else:
+            modified_files.append(file_path)
+        
         # Write file
-        path = Path(file_path)
         path.parent.mkdir(parents=True, exist_ok=True)
         path.write_text(file_content)
-        console.print(f"  [green]✓[/green] {file_path}")
+        
+        status = "[cyan]+ New[/cyan]" if is_new else "[green]✓[/green]"
+        console.print(f"  {status} {file_path}")
+        
+        # Check for new dependencies
+        if file_path == "requirements.txt":
+            new_deps = _detect_new_dependencies(file_content)
+        elif file_path == "pyproject.toml" and "[project]" in file_content:
+            new_deps = _detect_new_dependencies_pyproject(file_content)
+        
+        # Check for schema changes (model files)
+        if "models" in file_path.lower() and not is_new:
+            schema_changed = True
     
     console.print(f"\n[dim]Run 'engine rollback last' to undo these changes[/dim]")
+    
+    # Post-apply: Migration hints
+    if schema_changed:
+        console.print("\n[yellow]⚠️  Schema changes detected![/yellow]")
+        console.print("[dim]If using SQLite, you may need to:[/dim]")
+        console.print("  [cyan]rm *.db[/cyan]  (delete and recreate)")
+        console.print("[dim]Or if using Alembic:[/dim]")
+        console.print("  [cyan]alembic revision --autogenerate -m 'schema update'[/cyan]")
+        console.print("  [cyan]alembic upgrade head[/cyan]")
+    
+    # Post-apply: Auto-install dependencies
+    if new_deps:
+        console.print(f"\n[yellow]📦 New dependencies detected:[/yellow] {', '.join(new_deps)}")
+        if click.confirm("Install now?", default=True):
+            _install_dependencies(new_deps)
+    
+    # Post-apply: Git commit
+    if _is_git_repo():
+        if click.confirm("\n[dim]Commit changes to git?[/dim]", default=False):
+            _git_commit(task, file_paths)
+
+
+def _detect_new_dependencies(requirements_content: str) -> list[str]:
+    """Detect newly added dependencies in requirements.txt."""
+    req_path = Path("requirements.txt")
+    if not req_path.exists():
+        return []
+    
+    try:
+        old_deps = set()
+        for line in req_path.read_text().splitlines():
+            line = line.strip()
+            if line and not line.startswith("#"):
+                # Extract package name (before ==, >=, etc.)
+                pkg = line.split("==")[0].split(">=")[0].split("<=")[0].split("[")[0].strip()
+                old_deps.add(pkg.lower())
+        
+        new_deps = []
+        for line in requirements_content.splitlines():
+            line = line.strip()
+            if line and not line.startswith("#"):
+                pkg = line.split("==")[0].split(">=")[0].split("<=")[0].split("[")[0].strip()
+                if pkg.lower() not in old_deps:
+                    new_deps.append(pkg)
+        
+        return new_deps
+    except Exception:
+        return []
+
+
+def _detect_new_dependencies_pyproject(content: str) -> list[str]:
+    """Detect newly added dependencies in pyproject.toml."""
+    # Simplified detection - just look for new packages
+    return []  # TODO: Implement TOML parsing
+
+
+def _install_dependencies(deps: list[str]):
+    """Install dependencies using pip."""
+    import subprocess
+    
+    console.print(f"[dim]Installing: {' '.join(deps)}[/dim]")
+    try:
+        result = subprocess.run(
+            ["pip", "install"] + deps,
+            capture_output=True,
+            text=True
+        )
+        if result.returncode == 0:
+            console.print("[green]✓ Dependencies installed[/green]")
+        else:
+            console.print(f"[red]Installation failed:[/red] {result.stderr}")
+    except Exception as e:
+        console.print(f"[red]Failed to install:[/red] {e}")
+
+
+def _is_git_repo() -> bool:
+    """Check if current directory is a git repository."""
+    return (Path.cwd() / ".git").exists()
+
+
+def _git_commit(task: str, files: list[str]):
+    """Commit changes to git."""
+    import subprocess
+    
+    try:
+        # Add files
+        subprocess.run(["git", "add"] + files, capture_output=True)
+        
+        # Commit with task as message
+        commit_msg = f"engine: {task[:50]}" if len(task) > 50 else f"engine: {task}"
+        result = subprocess.run(
+            ["git", "commit", "-m", commit_msg],
+            capture_output=True,
+            text=True
+        )
+        
+        if result.returncode == 0:
+            console.print(f"[green]✓ Committed:[/green] {commit_msg}")
+        else:
+            console.print(f"[yellow]Git commit skipped:[/yellow] {result.stderr.strip()}")
+    except Exception as e:
+        console.print(f"[red]Git error:[/red] {e}")
 
 
 def _extract_file_changes(content: str) -> list[tuple[str, str]]:
@@ -1100,16 +1550,23 @@ def _extract_file_changes(content: str) -> list[tuple[str, str]]:
     
     changes = []
     
-    # Try new format first
-    new_pattern = r'<{5}\s*FILE:\s*(.+?)\s*>{5}\s*\n(.*?)\n<{5}\s*END FILE\s*>{5}'
-    new_matches = re.findall(new_pattern, content, re.DOTALL)
+    # Normalize line endings
+    content = content.replace('\r\n', '\n')
     
-    if new_matches:
-        for file_path, file_content in new_matches:
-            changes.append((file_path.strip(), file_content.strip()))
+    # Pattern: <<<<< FILE: path >>>>> ... <<<<< END FILE >>>>>
+    # More flexible - doesn't require specific whitespace/newlines
+    pattern = r'<{5}\s*FILE:\s*([^>]+?)\s*>{5}(.*?)<{5}\s*END\s*FILE\s*>{5}'
+    matches = re.findall(pattern, content, re.DOTALL)
+    
+    if matches:
+        for file_path, file_content in matches:
+            # Clean up the content
+            file_content = file_content.strip()
+            file_path = file_path.strip()
+            changes.append((file_path, file_content))
         return changes
     
-    # Fall back to legacy format
+    # Fall back to legacy format: FILE: path followed by code block
     legacy_pattern = r'^FILE:\s*(.+)$'
     parts = re.split(legacy_pattern, content, flags=re.MULTILINE)
     
@@ -1124,7 +1581,10 @@ def _extract_file_changes(content: str) -> list[tuple[str, str]]:
             if code_match:
                 file_content = code_match.group(1)
             
-            changes.append((file_path, file_content))
+            # Remove any trailing END FILE markers
+            file_content = re.sub(r'\s*<{5}\s*END\s*FILE\s*>{5}\s*$', '', file_content)
+            
+            changes.append((file_path, file_content.strip()))
         i += 2
     
     return changes
